import os
import subprocess
from distutils.version import LooseVersion
from glob import glob
from os.path import join

import setuptools.command.build_py
import setuptools.command.develop
from setuptools import Extension, find_packages, setup

version = "0.2.2"

# Adapted from https://github.com/py_torch/pytorch
cwd = os.path.dirname(os.path.abspath(__file__))
if os.getenv("PYSPTK_BUILD_VERSION"):
    version = os.getenv("PYSPTK_BUILD_VERSION")
else:
    try:
        sha = (
            subprocess.check_output(["git", "rev-parse", "HEAD"], cwd=cwd)
            .decode("ascii")
            .strip()
        )
        version += "+" + sha[:7]
    except subprocess.CalledProcessError:
        pass
    except IOError:  # FileNotFoundError for python 3
        pass


class build_py(setuptools.command.build_py.build_py):
    def run(self):
        self.create_version_file()
        setuptools.command.build_py.build_py.run(self)

    @staticmethod
    def create_version_file():
        global version, cwd
        print("-- Building version " + version)
        version_path = os.path.join(cwd, "pysptk", "version.py")
        with open(version_path, "w") as f:
            f.write("__version__ = '{}'\n".format(version))


class develop(setuptools.command.develop.develop):
    def run(self):
        build_py.create_version_file()
        setuptools.command.develop.develop.run(self)


cmdclass = {"build_py": build_py, "develop": develop}

min_cython_ver = "0.28.0"
try:
    import Cython

    ver = Cython.__version__
    _CYTHON_INSTALLED = ver >= LooseVersion(min_cython_ver)
except ImportError:
    _CYTHON_INSTALLED = False

try:
    if not _CYTHON_INSTALLED:
        raise ImportError("No supported version of Cython installed.")
    from Cython.Distutils import build_ext

    cython = True
except ImportError:
    cython = False
    from setuptools.command.build_ext import build_ext as _build_ext

    class build_ext(_build_ext):
        # https://stackoverflow.com/questions/19919905/how-to-bootstrap-numpy-installation-in-setup-py # noqa
        def finalize_options(self):
            _build_ext.finalize_options(self)
            # Prevent numpy from thinking it is still in its setup process:
            __builtins__.__NUMPY_SETUP__ = False
            import numpy

            self.include_dirs.append(numpy.get_include())


include_dirs = [join(os.getcwd(), "lib", "SPTK", "include")]
cmdclass["build_ext"] = build_ext
if cython:
    ext = ".pyx"
    import numpy as np

    include_dirs.insert(0, np.get_include())
else:
    ext = ".c"
    if not os.path.exists(join("pysptk", "_sptk" + ext)):
        raise RuntimeError("Cython is required to generate C code.")

# SPTK sources
src_top = join("lib", "SPTK")
src_bin_top = join(src_top, "bin")
swipe_src = [
    join(src_bin_top, "pitch", "swipe", "swipe.c"),
    join(src_bin_top, "pitch", "swipe", "vector.c"),
]
rapt_src = [
    join(src_bin_top, "pitch", "snack", "jkGetF0.c"),
    join(src_bin_top, "pitch", "snack", "sigproc.c"),
]

sptklib_src = glob(join(src_top, "lib", "*.c"))
sptk_src = glob(join(src_bin_top, "*", "_*.c"))

# collect all sources
sptk_all_src = sptk_src + sptklib_src + swipe_src + rapt_src

# Filter ignore list
ignore_bin_list = [
    join(src_bin_top, "wavjoin"),
    join(src_bin_top, "wavsplit"),
    join(src_bin_top, "vc"),
]
for ignore in ignore_bin_list:
    sptk_all_src = list(filter(lambda s, ig=ignore: not s.startswith(ig), sptk_all_src))

# define core cython module
ext_modules = [
    Extension(
        name="pysptk._sptk",
        sources=[join("pysptk", "_sptk" + ext)] + sptk_all_src,
        include_dirs=include_dirs,
        language="c",
        extra_compile_args=["-std=c99"],
    )
]

with open("README.md", "r") as fh:
    LONG_DESC = fh.read()

setup(
    name="pysptk",
    version=version,
    description="A python wrapper for Speech Signal Processing Toolkit (SPTK)",
    long_description=LONG_DESC,
    long_description_content_type="text/markdown",
    author="Ryuichi Yamamoto",
    author_email="zryuichi@gmail.com",
    url="https://github.com/r9y9/pysptk",
    license="MIT",
    packages=find_packages(exclude=["tests", "examples"]),
    package_data={"": ["example_audio_data/*"]},
    ext_modules=ext_modules,
    cmdclass=cmdclass,
    setup_requires=["numpy >= 1.20.0"],
    install_requires=[
        "scipy",
        "decorator",
        "cython >= " + min_cython_ver,
    ],
    tests_require=["pytest", "pytest-cov", "coverage"],
    extras_require={
        "docs": ["numpydoc", "sphinx_rtd_theme", "seaborn"],
        "test": ["pytest", "pytest-cov", "coverage", "flake8"],
        "lint": [
            "pysen",
            "types-setuptools",
            "mypy<=0.910",
            "black>=19.19b0,<=20.8",
            "click<8.1.0",
            "flake8>=3.7,<4",
            "flake8-bugbear",
            "isort>=4.3,<5.2.0",
            "types-decorator",
            "importlib-metadata<5.0",
        ],
    },
    classifiers=[
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Operating System :: MacOS",
        "Programming Language :: Cython",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
    ],
    keywords=["SPTK"],
)



import numpy as np
import pysptk
import pytest


def __test_filt_base(f, order, delay, *args):
    np.random.seed(98765)
    dummy_input = np.random.rand(1024)
    dummy_mgc = np.random.rand(order + 1)

    for x in dummy_input:
        assert np.isfinite(f(x, dummy_mgc, *args, delay=delay))
        assert np.all(np.isfinite(delay))


@pytest.mark.parametrize("order", [20, 25, 30])
def test_poledf(order):
    delay = pysptk.poledf_delay(order)
    __test_filt_base(pysptk.poledf, order, delay)
    __test_filt_base(pysptk.poledft, order, delay)


def test_poledf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.poledf(0.0, np.ones(10), np.ones(1))


@pytest.mark.parametrize("order", [20, 25, 30])
@pytest.mark.parametrize("pd", [4, 5])
def test_lmadf(order, pd):
    delay = pysptk.lmadf_delay(order, pd)
    __test_filt_base(pysptk.lmadf, order, delay, pd)


def test_lmadf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.lmadf(0.0, np.ones(10), 5, np.ones(1))


def test_lmadf_invalid_pade():
    with pytest.raises(ValueError):
        pysptk.lmadf(0.0, np.ones(10), 3, np.ones(1))


@pytest.mark.parametrize("order", [20, 25, 30])
def test_lspdf(order):
    delay = pysptk.lspdf_delay(order)
    __test_filt_base(pysptk.lspdf, order, delay)


def test_lspdf_invalid_delay_length():
    def __test(length):
        pysptk.lspdf(0.0, np.ones(length), np.ones(1))

    # even
    with pytest.raises(ValueError):
        __test(10)
    # odd
    with pytest.raises(ValueError):
        __test(9)


@pytest.mark.parametrize("order", [20, 25, 30])
def test_ltcdf(order):
    delay = pysptk.ltcdf_delay(order)
    __test_filt_base(pysptk.ltcdf, order, delay)


def test_ltcdf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.ltcdf(0.0, np.ones(10), np.ones(1))


@pytest.mark.parametrize("order", [20, 22, 25])
@pytest.mark.parametrize("stage", [1, 2, 3, 4, 5, 6])
def test_glsadf(order, stage):
    delay = pysptk.glsadf_delay(order, stage)
    __test_filt_base(pysptk.glsadf, order, delay, stage)
    __test_filt_base(pysptk.glsadft, order, delay, stage)


def test_glsadf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.glsadf(0.0, np.ones(10), 1, np.ones(1))


def test_glsadf_invalid_stage():
    with pytest.raises(ValueError):
        pysptk.glsadf(0.0, np.ones(10), 0, np.ones(1))


@pytest.mark.parametrize("order", [20, 25, 30])
@pytest.mark.parametrize("alpha", [0.0, 0.35, 0.5])
@pytest.mark.parametrize("pd", [4, 5])
def test_mlsadf(order, alpha, pd):
    delay = pysptk.mlsadf_delay(order, pd)
    __test_filt_base(pysptk.mlsadf, order, delay, alpha, pd)
    __test_filt_base(pysptk.mlsadft, order, delay, alpha, pd)


def test_mlsadf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.mlsadf(0.0, np.ones(10), 0.41, 5, np.ones(1))


def test_mlsadf_invalid_pade():
    with pytest.raises(ValueError):
        pysptk.mlsadf(0.0, np.ones(10), 0.41, 3, np.ones(1))


@pytest.mark.parametrize("order", [20, 25, 30])
@pytest.mark.parametrize("alpha", [0.0, 0.35, 0.5])
@pytest.mark.parametrize("stage", [1, 2, 3, 4, 5, 6])
def test_mglsadf(order, alpha, stage):
    delay = pysptk.mglsadf_delay(order, stage)
    __test_filt_base(pysptk.mglsadf, order, delay, alpha, stage)
    __test_filt_base(pysptk.mglsadft, order, delay, alpha, stage)


def test_mglsadf_invalid_delay_length():
    with pytest.raises(ValueError):
        pysptk.mglsadf(0.0, np.ones(10), 0.41, 15, np.ones(1))


def test_mglsadf_invalid_stage():
    with pytest.raises(ValueError):
        pysptk.mglsadf(0.0, np.ones(10), 0.41, 0, np.ones(1))



"""
Utilities
=========

Audio files
-----------

.. autosummary::
    :toctree: generated/

    example_audio_file


Mel-cepstrum analysis
---------------------

.. autosummary::
    :toctree: generated/

    mcepalpha
"""

# I originally tried with functools.wraps to create decoraters, but it didn't
# work to me if I use multiple decoratores to decorate a function.
# Specifically, I cannot inspect argspec with a decorated function, so cannot
# get a argment name simply from it. As suggested in the following
# stackoverflow thread, using decorator package.
# https://stackoverflow.com/questions/12558505/preserve-argspec-when-decorating
from inspect import getfullargspec

import numpy as np
import pkg_resources
from decorator import decorator

# 16kHz, 16bit example audio from cmu_us_awb_arctic
# see COPYING for the license of the audio file.
EXAMPLE_AUDIO = "example_audio_data/arctic_a0007.wav"


@decorator
def apply_along_last_axis(func, *args, **kwargs):
    """Apply function along last axis

    This is used for extending vector-to-vector operations to matrix-to-matrix
    operations. This basically does the following thing in a convenient way:

    ```py
    np.apply_along_axis(func, input_vector, -1, *args, **kwargs)
    ```

    Note: The decorator assumes that the first argment of the function is the
    input vector (1d numpy array).
    """

    # Get first arg
    first_arg_name = getfullargspec(func)[0][0]
    has_positional_arg = len(args) > 0
    input_arg = args[0] if has_positional_arg else kwargs[first_arg_name]

    if input_arg.ndim == 1:
        ret = func(*args, **kwargs)
    else:
        # we need at least 1 positonal argment
        if len(args) == 0:
            args = kwargs.pop(first_arg_name)
        ret = np.apply_along_axis(func, -1, *args, **kwargs)

    return ret


@decorator
def automatic_type_conversion(func, *args, **kwargs):
    first_arg_name = getfullargspec(func)[0][0]
    has_positional_arg = len(args) > 0
    input_arg = args[0] if has_positional_arg else kwargs[first_arg_name]
    dtypein = input_arg.dtype

    # Since C functions can only accept double
    if dtypein != np.float64:
        if has_positional_arg:
            args = tuple(
                map(
                    lambda v: input_arg.astype(np.float64) if v[0] == 0 else v[1],
                    enumerate(args),
                )
            )
        else:
            kwargs[first_arg_name] = input_arg.astype(np.float64)
    return func(*args, **kwargs).astype(dtypein)


@decorator
def automatic_type_conversion_float32(func, *args, **kwargs):
    first_arg_name = getfullargspec(func)[0][0]
    has_positional_arg = len(args) > 0
    input_arg = args[0] if has_positional_arg else kwargs[first_arg_name]
    dtypein = input_arg.dtype

    if dtypein != np.float32:
        if has_positional_arg:
            args = tuple(
                map(
                    lambda v: input_arg.astype(np.float32) if v[0] == 0 else v[1],
                    enumerate(args),
                )
            )
        else:
            kwargs[first_arg_name] = input_arg.astype(np.float32)
    return func(*args, **kwargs).astype(dtypein)


def assert_gamma(gamma):
    if not (-1 <= gamma <= 0.0):
        raise ValueError("unsupported gamma: must be -1 <= gamma <= 0")


def assert_pade(pade):
    valid = [4, 5, 6, 7]
    if pade not in valid:
        raise ValueError("4, 5, 6 or 7 pade approximation is supported")


def assert_stage(stage):
    if stage < 1:
        raise ValueError("stage >= 1 (-1 <= gamma < 0)")


def ispow2(num):
    return ((num & (num - 1)) == 0) and num != 0


def assert_fftlen(fftlen):
    if not ispow2(fftlen):
        raise ValueError("fftlen must be power of 2")


def example_audio_file():
    """Get the path to an included audio example file.

    Examples
    --------
    >>> from scipy.io import wavfile
    >>> fs, x = wavfile.read(pysptk.util.example_audio_file())

    >>> import matplotlib.pyplot as plt
    >>> plt.plot(x, label="cmu_us_awb_arctic arctic_a0007.wav")
    >>> plt.xlim(0, len(x))
    >>> plt.legend()

    """

    return pkg_resources.resource_filename(__name__, EXAMPLE_AUDIO)


def mcepalpha(fs, start=0.0, stop=1.0, step=0.001, num_points=1000):
    """Compute appropriate frequency warping parameter given a sampling frequency

    It would be useful to determine alpha parameter in mel-cepstrum analysis.

    The code is traslated from https://bitbucket.org/happyalu/mcep_alpha_calc.

    Parameters
    ----------
    fs : int
        Sampling frequency

    start : float
        start value that will be passed to numpy.arange. Default is 0.0.

    stop : float
        stop value that will be passed to numpy.arange. Default is 1.0.

    step : float
        step value that will be passed to numpy.arange. Default is 0.001.

    num_points : int
        Number of points used in approximating mel-scale vectors in fixed-
        length.

    Returns
    -------
    alpha : float
        frequency warping paramter (offen denoted by alpha)

    See Also
    --------
    pysptk.sptk.mcep
    pysptk.sptk.mgcep

    """
    alpha_candidates = np.arange(start, stop, step)
    mel = _melscale_vector(fs, num_points)
    distances = [
        rms_distance(mel, _warping_vector(alpha, num_points))
        for alpha in alpha_candidates
    ]
    return alpha_candidates[np.argmin(distances)]


def _melscale_vector(fs, length):
    step = (fs / 2.0) / length
    melscalev = 1000.0 / np.log(2) * np.log(1 + step * np.arange(0, length) / 1000.0)
    return melscalev / melscalev[-1]


def _warping_vector(alpha, length):
    step = np.pi / length
    omega = step * np.arange(0, length)
    num = (1 - alpha * alpha) * np.sin(omega)
    den = (1 + alpha * alpha) * np.cos(omega) - 2 * alpha
    warpfreq = np.arctan(num / den)
    warpfreq[warpfreq < 0] += np.pi
    return warpfreq / warpfreq[-1]


def rms_distance(v1, v2):
    d = v1 - v2
    return np.sum(np.abs(d * d)) / len(v1)



from os.path import dirname, join
from warnings import warn

import numpy as np
import pysptk
import pytest
from scipy.io import wavfile


@pytest.mark.parametrize("hopsize", [40, 80, 160, 320])
@pytest.mark.parametrize("otype", [0, 1, 2])
@pytest.mark.parametrize("otype_str", ["pitch", "f0", "logf0"])
def test_swipe(hopsize, otype, otype_str):
    def __test(x, fs, hopsize, otype):
        f0 = pysptk.swipe(x, fs, hopsize, otype=otype)
        assert np.all(np.isfinite(f0))
        if otype == 1:
            assert np.all(f0 >= 0)

    np.random.seed(98765)
    fs = 16000
    x = np.random.rand(16000)

    __test(x, fs, hopsize, otype)
    __test(x, fs, 80, otype_str)


def test_swipe_corner_case():
    def __test(x, fs, hopsize, otype):
        pysptk.swipe(x, fs, hopsize, otype=otype)

    np.random.seed(98765)
    fs = 16000
    x = np.random.rand(16000)

    # unsupported otype
    with pytest.raises(ValueError):
        __test(x, fs, 80, -1)
    with pytest.raises(ValueError):
        __test(x, fs, 80, 3)
    with pytest.raises(ValueError):
        __test(x, fs, 80, "ff0")


@pytest.mark.parametrize("hopsize", [40, 80, 160, 320])
@pytest.mark.parametrize("otype", [0, 1, 2])
@pytest.mark.parametrize("otype_str", ["pitch", "f0", "logf0"])
def test_rapt(hopsize, otype, otype_str):
    def __test(x, fs, hopsize, min, max, otype):
        f0 = pysptk.rapt(x, fs, hopsize, min=min, max=max, otype=otype)
        assert np.all(np.isfinite(f0))
        if otype == 1:
            assert np.all(f0 >= 0)

    np.random.seed(98765)
    fs = 16000
    x = np.random.rand(16000).astype(np.float32)

    __test(x, fs, hopsize, 60, 240, otype)
    __test(x, fs, 80, 60, 240, otype_str)


def test_rapt_corner_case():
    def __test(x, fs, hopsize, min, max, otype):
        f0 = pysptk.rapt(x, fs, hopsize, min=min, max=max, otype=otype)
        assert np.all(np.isfinite(f0))
        if otype == 1:
            assert np.all(f0 >= 0)

    np.random.seed(98765)
    fs = 16000
    x = np.random.rand(16000).astype(np.float32)

    # unsupported otype
    with pytest.raises(ValueError):
        __test(x, fs, 80, 60, 240, -1)
    with pytest.raises(ValueError):
        __test(x, fs, 80, 60, 240, 3)
    with pytest.raises(ValueError):
        __test(x, fs, 80, 60, 240, "f00")

    # valid min freq
    __test(x, fs, 80, 10, 240, 0)
    warn("TODO: fix RAPT bug to pass this minfreq lower bound test")
    # __test(x, fs, 80, fs / 10000. + 1, 240, 0)

    # valid max freq
    __test(x, fs, 80, 60, fs // 2 - 1, 0)

    # invalid min/max freq
    with pytest.raises(ValueError):
        __test(x, fs, 80, 60, 60, 0)
    with pytest.raises(ValueError):
        __test(x, fs, 80, 60, fs // 2, 0)
    with pytest.raises(ValueError):
        __test(x, fs, 80, fs / 10000.0, 240, 0)

    # valid frame_period (corner case)
    __test(x, fs, 1600, 60, 240, 0)
    __test(x, fs, 2, 60, 240, 0)

    warn("TODO: pass this corner case test")
    # __test(x, fs, 1, 60, 240, 0)

    # invalid frame_period
    with pytest.raises(ValueError):
        __test(x, fs, 1601, 60, 240, 0)

    # valid input length
    __test(x[:1000], fs, 80, 60, 240, 0)

    # invalid input length (too small)
    with pytest.raises(ValueError):
        __test(x[:100], fs, 80, 60, 240, 0)


def test_rapt_regression():
    # Grund truth data is generated by:
    #
    # $ wav2raw pysptk/example_audio_data/arctic_a0007.wav
    #
    # $ x2x +sf ./pysptk/example_audio_data/arctic_a0007.raw | \
    #    pitch -a 0 -s 16 -p 80 -L 60 -H 240 -o 0 > \
    #    arctic_a007_p16_L60_H240_o0_rapt.pitch
    #
    # $ dmp +f arctic_a007_p16_L60_H240_o0_rapt.pitch | awk '{print $2}' >\
    #    arctic_a007_p16_L60_H240_o0_rapt.txt
    #
    # $ pitch -h
    #  ...
    #
    #  SPTK: version 3.8
    #  CVS Info: $Id: pitch.c,v 1.46 2014/12/11 08:30:43 uratec Exp $

    ground_truth_path = join(
        dirname(__file__), "data", "arctic_a007_p16_L60_H240_o0_rapt.txt"
    )
    with open(ground_truth_path) as f:
        ground_truth = np.asarray([float(s) for s in [line for line in f.readlines()]])
    ground_truth = ground_truth.astype(np.float32)

    fs, x = wavfile.read(pysptk.util.example_audio_file())
    assert fs == 16000

    # Since SPTK might have memory corruption bug and the result might be
    # non-deterministic, test it with multiple time...
    for _ in range(5):
        f0 = pysptk.rapt(
            x.astype(np.float32),
            fs=fs,
            hopsize=80,
            min=60,
            max=240,
            voice_bias=0.0,
            otype=0,
        )
        assert np.allclose(ground_truth, f0)


def test_swipe_regression():
    # Grund truth data is generated by:
    #
    # $ wav2raw pysptk/example_audio_data/arctic_a0007.wav
    #
    # $ x2x +sf ./pysptk/example_audio_data/arctic_a0007.raw | \
    #    pitch -a 1 -s 16 -p 80 -L 60 -H 240 -o 0 > \
    #    arctic_a007_p16_L60_H240_o0_swipe.pitch
    #
    # $ dmp +f arctic_a007_p16_L60_H240_o0_swuoe.pitch | awk '{print $2}' >\
    #    arctic_a007_p16_L60_H240_o0_swipe.txt
    #
    # $ pitch -h
    #  ...
    #
    # SPTK: version 3.10
    # CVS Info: $Id: pitch.c,v 1.53 2016/12/25 05:00:19 uratec Exp $

    ground_truth_path = join(
        dirname(__file__), "data", "arctic_a007_p16_L60_H240_o0_swipe.txt"
    )
    with open(ground_truth_path) as f:
        ground_truth = np.asarray([float(s) for s in [line for line in f.readlines()]])
    ground_truth = ground_truth.astype(np.float32)

    fs, x = wavfile.read(pysptk.util.example_audio_file())
    assert fs == 16000

    # Since SPTK might have memory corruption bug and the result might be
    # non-deterministic, test it with multiple time...
    for _ in range(5):
        f0 = pysptk.swipe(
            x.astype(np.float64), fs=fs, hopsize=80, min=60, max=240, otype=0
        )
        assert np.allclose(ground_truth, f0)



import numpy as np
import pysptk
import pytest


def windowed_dummy_data(N):
    np.random.seed(98765)
    return pysptk.hanning(N) * np.random.randn(N)


# TODO: likely to have bugs in SPTK
@pytest.mark.parametrize("order", [20, 22, 25])
@pytest.mark.parametrize("pd", [4, 5])
def test_acep(order, pd):
    return
    # x = windowed_dummy_data(64)
    # c = np.zeros(order + 1)
    # for v in x:
    #     pysptk.acep(v, c, pd=pd)
    #     assert np.all(np.isfinite(c))


def test_acep_corner_case():
    def __test(order, pd):
        x = windowed_dummy_data(64)
        c = np.zeros(order + 1)
        for v in x:
            pysptk.acep(v, c, pd=pd)

    # invalid pade
    with pytest.raises(ValueError):
        __test(20, 3)
    with pytest.raises(ValueError):
        __test(20, 8)


@pytest.mark.parametrize("order", [20, 22, 25])
# TODO: likely to have bugs in SPTK
# @pytest.mark.parametrize("stage", [1, 2, 3, 4, 5, 6, 7, 8, 9])
@pytest.mark.parametrize("stage", [1, 2])
def test_agcep(order, stage):
    x = windowed_dummy_data(64)
    c = np.zeros(order + 1)
    for v in x:
        pysptk.agcep(v, c, stage=stage)
        assert np.all(np.isfinite(c))


def test_agcep_corner_case():
    def __test(order, stage):
        x = windowed_dummy_data(64)
        c = np.zeros(order + 1)
        for v in x:
            pysptk.agcep(v, c, stage=stage)

    # invalid stage
    with pytest.raises(ValueError):
        __test(20, 0)


@pytest.mark.parametrize("order", [20, 22, 25])
@pytest.mark.parametrize("alpha", [0.0, 0.35, 0.5])
def test_amcep(order, alpha, pd=5):
    x = windowed_dummy_data(64)
    c = np.zeros(order + 1)
    for v in x:
        pysptk.amcep(v, c, alpha=alpha, pd=pd)
        assert np.all(np.isfinite(c))


def test_amcep_corner_case():
    def __test(order, alpha, pd=5):
        x = windowed_dummy_data(64)
        c = np.zeros(order + 1)
        for v in x:
            pysptk.amcep(v, c, alpha=alpha, pd=pd)

    # invalid pade
    with pytest.raises(ValueError):
        __test(20, 0.35, 3)
    with pytest.raises(ValueError):
        __test(20, 0.35, 8)
